"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/* eslint-disable no-console */
const commander_1 = tslib_1.__importDefault(require("commander"));
const fs_1 = require("mz/fs");
const path_1 = require("path");
const index_1 = require("./index");
function run() {
    commander_1.default
        .description(`Sucrase: super-fast Babel alternative.`)
        .usage("[options] <srcDir>")
        .option("-d, --out-dir <out>", "Compile an input directory of modules into an output directory.")
        .option("--exclude-dirs <paths>", "Names of directories that should not be traversed.")
        .option("-t, --transforms <transforms>", "Comma-separated list of transforms to run.")
        .option("--enable-legacy-typescript-module-interop", "Use default TypeScript ESM/CJS interop strategy.")
        .option("--enable-legacy-babel5-module-interop", "Use Babel 5 ESM/CJS interop strategy.")
        .parse(process.argv);
    if (!commander_1.default.outDir) {
        console.error("Out directory is required");
        process.exit(1);
    }
    if (!commander_1.default.transforms) {
        console.error("Transforms option is required.");
        process.exit(1);
    }
    if (!commander_1.default.args[0]) {
        console.error("Source directory is required.");
        process.exit(1);
    }
    const outDir = commander_1.default.outDir;
    const srcDir = commander_1.default.args[0];
    const excludeDirs = commander_1.default.excludeDirs ? commander_1.default.excludeDirs.split(",") : [];
    const options = {
        transforms: commander_1.default.transforms.split(","),
        enableLegacyTypeScriptModuleInterop: commander_1.default.enableLegacyTypescriptModuleInterop,
        enableLegacyBabel5ModuleInterop: commander_1.default.enableLegacyBabel5ModuleInterop,
    };
    buildDirectory(srcDir, outDir, excludeDirs, options).catch((e) => {
        process.exitCode = 1;
        console.error(e);
    });
}
exports.default = run;
async function buildDirectory(srcDirPath, outDirPath, excludeDirs, options) {
    const extension = options.transforms.includes("typescript") ? ".ts" : ".js";
    if (!(await fs_1.exists(outDirPath))) {
        await fs_1.mkdir(outDirPath);
    }
    for (const child of await fs_1.readdir(srcDirPath)) {
        if (["node_modules", ".git"].includes(child) || excludeDirs.includes(child)) {
            continue;
        }
        const srcChildPath = path_1.join(srcDirPath, child);
        const outChildPath = path_1.join(outDirPath, child);
        if ((await fs_1.stat(srcChildPath)).isDirectory()) {
            await buildDirectory(srcChildPath, outChildPath, excludeDirs, options);
        }
        else if (srcChildPath.endsWith(extension)) {
            const outPath = `${outChildPath.substr(0, outChildPath.length - 3)}.js`;
            await buildFile(srcChildPath, outPath, options);
        }
    }
}
async function buildFile(srcPath, outPath, options) {
    console.log(`${srcPath} -> ${outPath}`);
    const code = (await fs_1.readFile(srcPath)).toString();
    const transformedCode = index_1.transform(code, Object.assign({}, options, { filePath: srcPath }));
    await fs_1.writeFile(outPath, transformedCode);
}
